body {
    font-family: "Lucida Grande", helvetica, arial, sans-serif;
    font-size: 0.9em;
    width: 700px;
}
strong {
    font-weight: bold;
}
em {
    font-style: italic;
}
a {
    color: #002286;
}
h1.title {
    font-size: 1.5em;
    width: 140px;
}
a:link {
    text-decoration: none;
}
a:visited {
    text-decoration: none;
}
.bluechips {
    color: #00a;
}
#nav {
    text-align: center;
}
#nav td {
    padding: 5px 15px;
}
table {
    border-collapse: collapse;
}
th, td {
    padding: 2px 5px;
}
div.block {
    margin: 10px 0;
    padding: 0 10px;
    border-bottom: 1px solid #bbb;
}
table.form {
    margin-bottom: 10px;
}
table.form th {
    text-align: right;
}
table.list {
    width: 100%;
    margin: 10px 0;
}
table.list th {
    text-align: left;
    border-bottom: 1px solid #bbb;
}

table.list th.amount {
    text-align: right;
}

table.list th.share {
    text-align: right;
}

table.list td {
    vertical-align: top;
}
table.list td.date {
    width: 90px;
}
table.list td.user {
    width: 110px;
}
table.list td.amount {
    width: 90px;
    text-align: right;
}
table.list td.share {
    width: 90px;
    text-align: right;
}
#totals {
    margin-bottom: 10px;
}
#totals th {
    text-align: right;
}
#totals th.scope {
    text-align: right;
    padding: 2px 15px;
}
#totals .scope {
    border-bottom: 1px solid #bbb;
}
#totals td {
    padding: 2px 15px;
    text-align: right;
}
#balance {
    margin-bottom: 10px;
}
#balance th {
    text-align: left;
}

#balance td.amount {
    text-align: right;
}
#balance th.amount {
    text-align: right;
}

td.labelcol {
    text-align: right;
    font-weight: bold;
}
tr.user-not-involved {
    color: #aaa;
}
div.flash {
    margin: 10px 0;
    padding: 0.5em;
    font-weight: bold;
    background: #fffa98;
    border: 1px solid #e6e000;
}
div.flash ul {
    margin-bottom: 0;
}
span.error-message {
    font-weight: bold;
    color: #f00;
}
input.currency {
    padding-left: 10px;
    background-image: url(../icons/currency-small.png);
    background-repeat: no-repeat;
    background-position: center left;
}
span.see-all {
    font-size: 0.7em;
}
#mobile {
    padding: 5px;
    font-weight: bold;
    border-bottom: 1px solid #bbb;
}
